use warnings;
use strict;

#-----------------------------------------------------------------------------
# There exists at least one case in which a XML::Twig->parse() is not round
# trip safe. For our purpose round trip safe is defined as:
#
#     my $first_data = XML::Twig->new(%params)->root()->parse($xml)->sprint();
#     my $second_data = XML::Twig->new(%params)->root()->parse($first_data)->sprint();
#
#     Test::More::is($first_data, $xml);
#     Test::More::is($second_data, $xml);
#
# what we have found is that if %params = ( keep_encoding => 1 ), and needs to be so
# because the xml data read from disk may have been saved with special characters,
# such as a Bullet - or the file saved in a file format other than utf-8, that it is
# possible to cause a failure in what would other wise be a round trip.
#
# There is at least one XML element <Chapter />, which, if it is processed by
# an XML::Twig, without the keep_encoding set, then any subsequent call to process
# XML that contains special characters, such as a Bullet, will fail - as the Bullet
# will be converted into the character string 'â'
#
#
#
#-----------------------------------------------------------------------------

# IF the parse is of Chapter, then the test_roundTripSafe will fail.
#$causesFailureTwig->parse('<Chapter></Chapter>');
# OH NO... if Chapter is an element in the parse...
#$causesFailureTwig->parse('<container><Chapter></Chapter></container>');
# the plural will not fail
# $causesFailureTwig->parse('<Chapters></Chapters>');
# lower case chapter will not cause the failure.
#$causesFailureTwig->parse('<chapter></chapter>');
#  without the full spelling, of Chapter, the failure does not occur.
#$causesFailureTwig->parse('<Chapte></Chapte>');

use English qw(-no_match_vars);
use FindBin qw ($Bin);
use Carp;
use Test::More qw(no_plan);
use XML::Twig;

use File::Spec;

use constant TEST_DATA_DIRECTORY => File::Spec->catdir( $Bin, 'test_data', );

#-----------------------------------------------------------------------------

roundTripSafe('test');

exit;

#-----------------------------------------------------------------------------

sub roundTripSafe {

    my ($tag) = @_;

    # this will only work IF the 'keep_encoding' is turned off.
    my @fileNames = qw(heading.xml);

    my %params = ( keep_encoding => 1 );

    foreach my $fileName (@fileNames) {
        my $file_path = File::Spec->join( TEST_DATA_DIRECTORY, $fileName );

        my $xml = stringFromFile($file_path);
        chomp $xml;
        my $twig = XML::Twig->new(%params);
        $twig->parse($xml);

        my $twig_root       = $twig->root();
        my $want_round_trip = $twig_root->sprint();

        my $clean_xml = "$xml";
        $clean_xml =~ s{<\?xml.*\?>[\r\n]*}{};
        Test::More::diag("xml: '$clean_xml'");

        Test::More::is( $want_round_trip, $clean_xml,
            "roundTripSafe - xml from File equals sprint from Twig" );

        my $twig2 = XML::Twig->new(%params);
        $twig2->parse($want_round_trip);

        my $got_xml = $twig2->root()->sprint();

        Test::More::is( $got_xml, $clean_xml,
            "roundTripSafe - $fileName after processing '$tag'" );
    }

    return 1;
}

#-----------------------------------------------------------------------------

sub stringFromFile {

    # Reads file into a textRef instead of breaking up into lines.
    my ( $path, $mode ) = @_;

    $mode = '<';
    local $/;
    open( FILE, $mode, $path )
        or Carp::confess("stringRefFromFile: can't open $path. $OS_ERROR");
    my $fileContents = <FILE>;
    close FILE;

    return $fileContents;
}

#-----------------------------------------------------------------------------

